from .app import *
from .channel import *
from .type import *
from .alias import *

class ControllerClient:
    def __init__(self, insprd_url, scope) -> None:
        self.app = AppClient(insprd_url, scope)
        self.channel = ChannelClient(insprd_url, scope)
        self.type = TypeClient(insprd_url, scope)
        self.alias = AliasClient(insprd_url, scope)


import sys
from ..models import *
from ..rest import *
from .client import *

APP_ROUTE = "apps"

class AppClient(Client):
    def get(self, scope:str) -> InsprStructure:
        headers = self.get_header_with_scope(scope)

        try:
            resp = send_get_request(self.url + "/" + APP_ROUTE, headers=headers)
            return InsprStructure(json.loads(resp.text))
        
        except Exception as e:
            raise Exception(f"Error while send a Get App request: {e}")

    def delete(self, scope:str, dryRun:bool) -> Changelog:
        msg_body = {
            "dry": dryRun
        }

        headers = self.get_header_with_scope(scope)

        try:
            resp = send_delete_request(self.url + "/" + APP_ROUTE, body=msg_body, headers=headers)
            return Changelog(json.loads(resp.text))
        except Exception as e:
            raise Exception(f"Error while send a Delete App request: {e}")

    def create(self, scope:str, app:dict, dryRun:bool) -> Changelog:
        msg_body = {
            "app": app,
            "dry": dryRun
        }

        headers = self.get_header_with_scope(scope)

        try:
            resp = send_post_request(self.url + "/" + APP_ROUTE, msg_body, headers)
            return Changelog(json.loads(resp.text))
        except Exception as e:
            raise Exception(f"Error while send a Create App request: {e}")

    def update(self, scope:str, app:dict, dryRun:bool) -> Changelog:
        msg_body = {
            "app": app,
            "dry": dryRun
        }

        headers = self.get_header_with_scope(scope)

        try:
            resp = send_update_request(self.url + "/" + APP_ROUTE, msg_body, headers)
            return Changelog(json.loads(resp.text))
        except Exception as e:
            raise Exception(f"Error while send a Update App request: {e}")


import json

class Changelog:
    def __init__(self, changelog:dict) -> None:
        self.obj = changelog
    
    def __str__(self) -> str:
        return f'Changelog({ json.dumps(self.obj, sort_keys=False, indent=4) })'

class InsprStructure:
    def __init__(self, structure:dict) -> None:
        self.obj = structure
    
    def __str__(self) -> str:
        return f'InsprStructure({ json.dumps(self.obj, sort_keys=False, indent=4) })'


import sys
from ..models import *
from ..rest import *
from .client import *

CHANNEL_ROUTE = "channels"

class ChannelClient(Client):
    def get(self, scope:str, channel_name:str) -> InsprStructure:
        msg_body = {
            "chname": channel_name
        }
        
        headers = self.get_header_with_scope(scope)

        try:
            resp = send_get_request(self.url + "/" + CHANNEL_ROUTE, body=msg_body, headers=headers)
            return InsprStructure(json.loads(resp.text))
        except Exception as e:
            raise Exception(f"Error while send a Get Channel request: {e}")

    def delele(self, scope:str, channel_name:str, dryRun:bool) -> Changelog:
        msg_body = {
            "chname": channel_name,
            "dry": dryRun
        }

        headers = self.get_header_with_scope(scope)

        try:
            resp = send_delete_request(self.url + "/" + CHANNEL_ROUTE, body=msg_body, headers=headers)
            return Changelog(json.loads(resp.text))
        except Exception as e:
            raise Exception(f"Error while send a Delete Channel request: {e}")

    def create(self, scope:str, channel:dict, dryRun:bool) -> Changelog:
        msg_body = {
            "channel": channel,
            "dry": dryRun
        }

        headers = self.get_header_with_scope(scope)

        try:
            resp = send_post_request(self.url + "/" + CHANNEL_ROUTE, msg_body, headers)
            return Changelog(json.loads(resp.text))
        except Exception as e:
            raise Exception(f"Error while send a Create Channel request: {e}")

    def update(self, scope:str, channel:dict, dryRun:bool) -> Changelog:
        msg_body = {
            "channel": channel,
            "dry": dryRun
        }

        headers = self.get_header_with_scope(scope)

        try:
            resp = send_update_request(self.url + "/" + CHANNEL_ROUTE, msg_body, headers)
            return Changelog(json.loads(resp.text))
        except Exception as e:
            raise Exception(f"Error while send a Update Channel request: {e}")


from setuptools import setup

with open("README.md", "r") as fh:
    long_description = fh.read()

setup(
    name="inspr",
    version="0.0.11",
    author="Inspr LLC",
    description="This module define APIs to interact with the Inspr environment",
    long_description=long_description,      # Long description read from the the readme file
    long_description_content_type="text/markdown",
    python_requires='>=3.8',
    packages=[
        "inspr",
        "inspr.controller"
    ],
    requires=[
        "requests",
        "flask"
    ],
    install_requires=[
        "requests",
        "flask"
    ]
)



import requests
import json
import sys
from flask.wrappers import Response
from requests.models import HTTPError

def send_post_request(url:str, body = {}, headers = {}) -> Response:
    try:
        resp = requests.post(url, data = json.dumps(body), headers=headers)
        resp.raise_for_status()
    
    except requests.exceptions.RequestException as e:
        print(e, file=sys.stderr)
        raise HTTPError
    
    return resp

def send_update_request(url:str, body = {}, headers = {}) -> Response:
    try:
        resp = requests.put(url, data = json.dumps(body), headers=headers)
        resp.raise_for_status()
    
    except requests.exceptions.RequestException as e:
        print(e, file=sys.stderr)
        raise HTTPError
    
    return resp

def send_get_request(url:str, body = {}, headers = {}) -> Response:
    try:
        resp = requests.get(url, data=json.dumps(body), headers=headers)
        resp.raise_for_status()
    
    except requests.exceptions.RequestException as e:
        print(e, file=sys.stderr)
        raise HTTPError
    
    return resp

def send_delete_request(url:str, body = {}, headers = {}) -> Response:
    try:
        resp = requests.delete(url, data=json.dumps(body), headers=headers)
        resp.raise_for_status()
    
    except requests.exceptions.RequestException as e:
        print(e, file=sys.stderr)
        raise HTTPError
    
    return resp



import os

INSPRD_ADDRESS = "INSPR_INSPRD_ADDRESS"
SCOPE = "INSPR_CONTROLLER_SCOPE"
TOKEN = "INSPR_CONTROLLER_TOKEN"
TOKEN_PATH = ".inspr/token"

class Client:
    def __init__(self, insprd_url, global_scope = "") -> None:
        self.url = insprd_url
        self.global_scope = global_scope
    
    def get_header_with_scope(self, scope:str) -> dict:
        headers = {
            "content-type": "application/json",
            "Scope": self.join_scopes(self.global_scope, scope),
            "Authorization": self.get_token() 
        }
        return headers

    def get_token(self) -> str:
        home_dir = os.path.expanduser('~')
        token_path = os.path.join(home_dir, TOKEN_PATH)
        f = open(token_path, "r")
        token = f.read().strip()

        if token is None:
            raise Exception("cant get token")
        
        token = "Bearer " + token
        return token

    def join_scopes(self, s1:str, s2:str) -> str:
        if s2 == "":
            return s1
        
        separator = ""
        if s1 != "":
            separator = "."
        
        new_scope = s1 + separator + s2
        return new_scope